#!/usr/bin/perl -w
use strict;
use IO::Socket;
use IO::Socket::SSL;
use Getopt::Std;

###
# SDS Backupscript
# Backup des kompletten SDS Archivs im Batchmodus
#
# Author:	bs@softmine.net
# Created:	25.09.2008
# Update:       07.09.2010 pe@softmine.net
###

#CONFIGURATION
my $ADMINDHOST = 'localhost';	#Host des AdminDeamon (default: localhost)
my $ADMINDPORT = '23020';		#Post des AdminDeamon (default: 23027)
my $ADMINDSSL = '1';			#Verwendet AdminDeamon SSL? (1 true, 0 false)

############################################################
# End of user configurable parts
############################################################

## Nicht ndern ab hier
my $SYSID;
my $MOUNT;
my $ISO;
my $USER;
my $PASS;
my $MPASS;
my $JOBID;
my $OUTFILE;
my $WHERE = "\" WHERE 1=1\"";
my $progname = $0;
$progname =~ s#.*/(.*?)$#$1#;

##Forward Declarations
sub usage($);
sub error($@);
sub connectAD();

######### MAIN ##########
my %opt = ();

getopts('u:x:s:o:m:hp:', \%opt);
usage(0)  if ($opt{h});
if (!(defined($opt{s}) and defined($opt{m}) and defined($opt{u}) and defined($opt{x})  )) {
	usage(1);
} 

#Parameter holen
$SYSID = $opt{s};
$MOUNT = $opt{m};
$OUTFILE = $opt{o};
$USER = $opt{u};
$PASS = $opt{x};
$MPASS = $opt{p};


#Connect AD
my $ads = connectAD();
#Kommando absetzen
my $cmd = "AUTH $SYSID $USER $PASS;BACKUP -e 9999 ";
$cmd.= "-o $OUTFILE " if(defined($opt{o}));
if(defined($MPASS))
{
	$cmd .= " -p " . $MPASS 
}
$cmd .= " ";
$cmd .= $MOUNT;
$cmd .= " ";
$cmd .= $WHERE;
$cmd .= ";QUIT\n";

print $ads $cmd;
while ( <$ads> ) { 
	my $line = $_;
	if($line =~ /^O:.*/)
	{
		$line =~ s/O://g;
		$JOBID = $line;
		chomp($JOBID);
	}
}
#Verbindung schlieen
close($ads);

#So lange pollen bis Status erhalen
if(!defined($JOBID))
{
	error(1, "Es wurde kein Job erstellt");
}

my $bstatus;
while(1)
{
	$ads = connectAD();
	$cmd = "AUTH $SYSID $USER $PASS;LIST_JOB ";
	$cmd .= $JOBID;
	$cmd .= ";QUIT\n";
	
	print $ads $cmd;
	while ( <$ads> ) { 
		my $line = $_;
		if($line =~ /^S:ERROR/)
		{
			close($ads);
			error(1, "Fehler beim Erstellen des Backups");
		} 
		
		if($line =~ /^S:DONE/) {
			close($ads);
			print "BACKUP OK";
			exit(0);
		}
	}
	sleep(1);
	#Verbindung schlieen
	close($ads);
}

#Funktionen
sub connectAD()
{
	my $ad;
	if($ADMINDSSL == 1)
	{
		$ad = IO::Socket::SSL->new(	Proto     => 'tcp',
	                				PeerAddr  => $ADMINDHOST,
	                				PeerPort  => $ADMINDPORT,
	                				SSL_use_cert => 0,
	                				SSL_verify_mode => 0x00,
	                				SSL_version => 'SSLv3',
	                			)
		|| die "Es konnte keine Verbindung zum AdminDeamon hergestellt werden: " .IO::Socket::SSL::errstr();
		
	} else {
		$ad = IO::Socket::INET->new(
	                Proto     => 'tcp',
	                PeerAddr  => $ADMINDHOST,
	                PeerPort  => $ADMINDPORT,
	   ) || die "Es konnte keine Verbindung zum AdminDeamon hergestellt werden";
	}
	return $ad;
}

sub usage($)
{
	my $status = shift;

	print STDERR "Usage: $progname [<options>]\n";
	if (!$status) {
	print STDERR "  <options> sind\n" .
	             "    -u <user>                 Name des Users, der ein Backup erstellen darf\n" .
	             "    -x <pass>                 Passwort des Users\n" .
	             "    -s <sysid>                Backup fuer <sysid> schreiben\n" .
	             "    -m <mount>                Mount fuer das Backup in der Form nfs://server/share oder CIFS\n" .
	             "    -o <outfile>              Name der backup Datei\n" .
	             "    -p <pass>                 CIFS Passwort\n" ;
	} else {
		print STDERR "Hilfe: $progname -h\n";
	}  
	
 	exit($status);
}

sub error($@)
{
	my $status = shift;
	my @msg = @_;
	print STDERR $progname . ": " . join(" ", @msg) . "\n";
	exit($status)  if ($status);
}
